/* 
* Rootfly:  A program for analyzing and measuring roots. 
* Copyright (c) 2007 Clemson University.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

// Initial controller implementation

#pragma warning(disable:4786)  //Added to remove compiler warning


#include "Controller.h"

extern DebugLogFile g_debug_logfile;

/**
@brief Attaches an Observer to the Subject
@params o - The observer to be attached
*/
void Subject::Attach (Observer* o) {
  m_observers.push_back(o);
}

/**
@brief Detaches an observer from the subject
@params o - The observer to be detached
*/
void Subject::Detach (Observer* o) {
  int count = m_observers.size();
  int i;
  
  /// Find the observer to detach
  for (i = 0; i < count; i++)
  {
    if(m_observers[i] == o)
      break;
  }
  
  /// Remove the observer from the vector so it will not recieve notifications
  if(i < count)
    m_observers.erase(m_observers.begin() + i);
  
}

/**
@brief Calls OnUpdate with the input notification code on all observers
@params code_notify - The notification code to be sent
*/
void Subject::Notify (int code_notify) {
  int count = m_observers.size();
  
  /// Update all observers
  for (int i = 0; i < count; i++)
    (m_observers[i])->OnUpdate(this, code_notify); 
  
}

/**
@brief Wrapper function for CreateNewProject
@params filename - name of the rootfly file
@params directory - The image directory
*/
void Controller::C_CreateNewProject(const CString& filename, const CString& directory)
{
  /// Get information from imagefileformat to name and create various data structures
  ImageFileFormat temp_file_format = Controller::C_GetImageFileFormat();
  ImageInfo temp_image_info = temp_file_format.GetImageInfoOfFirstFile(directory);

  // Controller::C_SetOperator(temp_image_info.initials); // should be okay to put this back in, but at one point it was causing program to crash, and it is not important at all because no one uses the initials anyway -- STB
  Controller::C_SetFilename(filename);
  Controller::C_SetModify(false);
  Controller::C_SetExperiment(temp_image_info.expt);
  Controller::C_SetTube(temp_image_info.tube);
  Controller::C_SetWindow(temp_image_info.window);
  Controller::C_SetSession(temp_image_info.session);
  
  Notify(ON_INIT);
}



/**
@brief Wrapper function for LoadFile
@param fname - the name of the file to be loaded
@returns 0 for success, nonzero for failure (see the ERRORS enum in DataStructures.h)
*/
int Controller::C_LoadFile(const CString& fname)
{
  int temp_int;
  Experiment *exp = m_data.GetExperiment();
  
  exp->SetOldFile(false);
  Settings *settings = m_data.GetSettings();
  
  /// Load file and set the filename in the data structures
  temp_int = exp->Load((char *) (LPCTSTR)fname,settings);
  if (temp_int == ERRORS::RFY_LOADERROR_SUCCESS)
  {
    Controller::C_SetFilename(fname);
  
    /// Set status saved
    Controller::C_SetModify(false);
    Notify(NEW_FILE_LOADED);
  }
  
  return temp_int;
}

/**
@brief Wrapper function for SaveFile
@param fname - Name of the file to be saved
@returns 0 for sucess, -1 for failure
*/
int Controller::C_SaveFile(const CString& fname)
{
  int temp_int;
  g_debug_logfile.Print("Controller::C_SaveFile 1\n"); 
  Experiment *exp = m_data.GetExperiment();
  g_debug_logfile.Print("Controller::C_SaveFile 2 %x\n", exp); 
  Settings *settings = m_data.GetSettings();
  g_debug_logfile.Print("Controller::C_SaveFile 3 %x\n", settings); 
  
  /// Save the file in the experiment
  temp_int = exp->Save((char *) (LPCTSTR)fname, settings);
  
  g_debug_logfile.Print("Controller::C_SaveFile 4\n"); 
  Notify(FILE_SAVED);
  g_debug_logfile.Print("Controller::C_SaveFile 5\n"); 
  /// Set status saved
  Controller::C_SetModify(false);
  g_debug_logfile.Print("Controller::C_SaveFile 6\n"); 
  
  return temp_int;
}

/**
@brief Returns RootInstance from root_id based on current ApplicationData
@param root_id Identification number for the root
@return Pointer to RootInstance
*/
RootInstance *Controller::C_GetRootInstance(int root_id)
{
  /// Get the current tube and session number from the ApplicationData
  ApplicationData* app_data = m_data.GetApplicationData();
  CString tube_tag = app_data->GetTube();
  int session = app_data->GetSession();
  
  /// Get the tube from the experiment using the tag from the ApplicationData
  Experiment *exp = m_data.GetExperiment();
  Tube* tube = exp->GetTube(tube_tag);
  /// Failure if tube is null
  if(tube == NULL)
  {
    return NULL;
  }
  
  /// Get the root from the tube by the Root ID
  Root* temp_root = tube->GetRoot(root_id);
  /// Failure if root is null
  if(temp_root == NULL)
  {
    return NULL;
  }
  
  /// Get the root instance using the session number from ApplicationData
  RootInstance* root_instance = temp_root->GetRootInstance(session);
  
  return root_instance;
}

/**
@brief Returns RootInstance from root_id based on current ApplicationData
@param root_id Identification number for the root
@return Pointer to RootInstance
*/
RootInstance *Controller::C_GetRootInstance(int root_id, int session)
{
  /// Get the current tube and session number from the ApplicationData
  ApplicationData* app_data = m_data.GetApplicationData();
  CString tube_tag = app_data->GetTube();  
  
  /// Get the tube from the experiment using the tag from the ApplicationData
  Experiment *exp = m_data.GetExperiment();
  Tube* tube = exp->GetTube(tube_tag);
  /// Failure if tube is null
  if(tube == NULL)
  {
    return NULL;
  }
  
  /// Get the root from the tube by the Root ID
  Root* temp_root = tube->GetRoot(root_id);
  /// Failure if root is null
  if(temp_root == NULL)
  {
    return NULL;
  }
  
  /// Get the root instance using the session number from ApplicationData
  RootInstance* root_instance = temp_root->GetRootInstance(session);
  
  return root_instance;
}

/**
@brief Returns RootInstance from root_id based on current ApplicationData
@param root_id Identification number for the root
@return Pointer to RootInstance
*/
RootInstance *Controller::C_CreateRootInstance(int root_id)
{
  ApplicationData* app_data = m_data.GetApplicationData();
  CString tube_tag = app_data->GetTube();
  int session = app_data->GetSession();
  
  Experiment *exp = m_data.GetExperiment();
  Tube* tube = exp->GetTube(tube_tag);
  /// Failure if tube is null
  if(tube == NULL)
  {
    return NULL;
  }
  assert(tube);
  
  Root* temp_root = tube->GetRoot(root_id);
  /// Failure if root is null
  if(temp_root == NULL)
  {
    return NULL;
  }
  assert(temp_root);
  
  RootInstance* root_instance = temp_root->CreateRootInstance(session);
  
  /// Set status unsaved
  Controller::C_SetModify(true);
  Notify(ROOT_INSTANCE_ADDED);
  return root_instance;
}

/**
@brief Wrapper function for CreatePoint
@params root_id - index of root instance to which the point is created
@returns Pointer to RPoint created
*/
RPoint* Controller::C_CreatePoint(int root_id)
{
  RPoint *temp_point;
  RootInstance *root_inst = C_GetRootInstance(root_id);
  
  /// Failure if root instance is null
  if(root_inst == NULL)
  {
    return NULL;
  }
  
  temp_point = root_inst->CreatePoint();
  Notify(POINT_ADDED);
  /// Set status unsaved
  Controller::C_SetModify(true);
  return temp_point;
}

/**
@brief Wrapper function for DeletePoint
@params index - index of point to be deleted
@params root_id - index of root instance from which point will be deleted
@returns true for sucess, false for failure
*/
bool Controller::C_DeletePoint(int index, int root_id)
{
  bool temp_bool;
  RootInstance *root_inst = C_GetRootInstance(root_id);
  
  /// Failure if root instance is null
  if(root_inst == NULL)
  {
    return NULL;
  }
  
  temp_bool = root_inst->DeletePoint(index);  
  Notify(POINT_REMOVED);
  /// Set status unsaved
  Controller::C_SetModify(true);
  
  return temp_bool;
}

/**
@brief Function to move all roots in current Tube Window Session
@params window - window to move points in
@params offset_x - x coordinate offset of moved point
@params offset_y - y coordinate offset of moved point
@returns true for sucess, false for failure
*/
bool Controller::C_MoveRootsInWindow(int window,int offset_x, int offset_y){
  
  //get all roots in tube/window/session
  Experiment *exp = m_data.GetExperiment();
  Tube *tube = exp->GetTube(m_data.GetApplicationData()->GetTube());
  /// Failure if tube is null
  if( tube == NULL ) return false;
  
  /// Get all roots
  vector<Root> *roots = tube->GetRootsInWindow(window);
  if (roots)
  {
    vector<Root>::iterator iter;
    //for each root
    for(iter = roots->begin();iter != roots->end();iter++)
      C_MoveRootSess(iter->GetID(),m_data.GetApplicationData()->GetSession(), offset_x, offset_y);
  }
  
  return true;
}

/**
@brief Function to move a root for a particular session
@params root_id - index of root to be moved
@params session - session in which to move the root
@params offset_x - x coordinate offset of moved point
@params offset_y - y coordinate offset of moved point
@returns true for sucess, false for failure
*/
bool Controller::C_MoveRootSess(int root_id, int session, int offset_x, int offset_y)
{
  Experiment *exp = m_data.GetExperiment();
  Tube *tube = exp->GetTube(m_data.GetApplicationData()->GetTube());
  /// Failure if tube is null
  if( tube == NULL ) return false;
  
  Root *r_root = tube->GetRoot(root_id);
  /// Failure if root is null
  if( r_root == NULL ) return false;
  
  RootInstance *r_inst = r_root->GetRootInstance(session);
  /// Failure if root instance is null
  if( r_inst == NULL ) return false;
  
  /// For all points
  for(int i = 0;i<r_inst->GetAllPoints()->size();i++)
    C_MovePoint(i, root_id, r_inst->GetPoint(i)->x+offset_x,r_inst->GetPoint(i)->y+offset_y);//may need to update this to be a bit different with x and y
  
  /// Move Diameter (assumes only one diameter)
  C_MoveDiameter(0,root_id, r_inst->GetDiameter(0)->r_point.x + offset_x, r_inst->GetDiameter(0)->r_point.y + offset_y);
  
  return true;
}

/**
@brief Wrapper function for MovePoint
@param index - index of point to be moved
@params root_id - index of root to be moved
@params offset_x - x coordinate offset of moved point
@params offset_y - y coordinate offset of moved point
@returns true for sucess, false for failure
*/
bool Controller::C_MovePoint(int index, int root_id, int x, int y)
{ 
  RootInstance *root_inst = C_GetRootInstance(root_id);
  /// Failure if root instance is null
  if(root_inst == NULL)     return false;
  
  /// Move the point
  if(root_inst->MovePoint(index, x, y))
  {
    /// Set status unsaved
    Controller::C_SetModify(true);
    Notify(POINT_MOVED);
    return true;
  }
  else return false;
}

/**
@brief Wrapper function for MovePoint
@param index - index of point to be moved
@params root_id - index of root to be moved
@params offset_x - x coordinate offset of moved point
@params offset_y - y coordinate offset of moved point
@returns true for sucess, false for failure
*/
void Controller::C_MovePointSess(int index, int root_id, int session, int x, int y)
{ 
  RootInstance *root_inst = C_GetRootInstance(root_id, session);
  /// Failure if root instance is null
  if(root_inst == NULL)     return;
  
  /// Move the point
  root_inst->MovePoint(index, x, y);  
}

/**
@brief The is a wrapper for ChangeWindowOfPoint
@params root_id - index of root instance to which point belongs
@params index - index of point whose window is to be changed
@params w - window to move point to
@returns true for sucess, false for failure
*/
bool Controller::C_ChangeWindowOfPoint(int index, int root_id, int w)
{
  RootInstance *root_inst = C_GetRootInstance(root_id);
  /// Failure if root instance is null
  if(root_inst == NULL)     return false;
  
  if(root_inst->ChangeWindowOfPoint(index, w))
  {
    /// Set status unsaved
    Controller::C_SetModify(true);    
    return true;
  }
  else return false;
}

/**
@brief Wrapper function for CreateDiameter
@params root_id - index of root instance to which diameter will be added
@returns Pointer to created RDiameter
*/
RDiameter* Controller::C_CreateDiameter(int root_id)
{
  RDiameter *temp_diam;
  RootInstance *root_inst = C_GetRootInstance(root_id);
  
  /// Failure if root instance is null
  if(root_inst == NULL)
  {
    return NULL;
  }
  
  temp_diam = root_inst->CreateDiameter();
  Notify(DIAMETER_ADDED);
  /// Set status unsaved
  Controller::C_SetModify(true);
  return temp_diam;
}

/**
@brief Wrapper function for DeleteDiameter
@params root_id - index of root to which diameter belongs
@params index - index of diameter to be deleted
@returns true for sucess, false for failure
*/
bool Controller::C_DeleteDiameter(int index, int root_id)
{
  bool temp_bool;
  RootInstance *root_inst = C_GetRootInstance(root_id);
  
  /// Failure if root instance is null
  if(root_inst == NULL)
  {
    return NULL;
  }
  
  temp_bool = root_inst->DeleteDiameter(index);
  Notify(DIAMETER_REMOVED);
  /// Set status unsaved
  Controller::C_SetModify(true);
  return temp_bool;
}

/**
@brief Wrapper function for MoveDiameter
@params index - index of diameter to be moved
@params root_id - index of root to be moved
@params x - x coordinate offset of moved diameter
@params y - y coordinate offset of moved diameter
@returns true for sucess, false for failure
*/
bool Controller::C_MoveDiameter(int index, int root_id, int x, int y)
{  
  RootInstance *root_inst = C_GetRootInstance(root_id);
  
  /// Failure if root instance is null
  if(root_inst == NULL)     return false;
  
  if(root_inst->MoveDiameter(index, x, y))
  {
    /// Set status unsaved
    Controller::C_SetModify(true);
    Notify(DIAMETER_MOVED);
    return true;
  }
  else return false;
}

bool Controller::C_MoveDiameter(int index, int root_id, const RPoint& r)
{
  bool success1, success2;
  success1 = C_MoveDiameter(index, root_id, r.x, r.y); 
  success2 = C_ChangeWindowOfDiameter(index, root_id, r.window);
  return success1 && success2;
}

/**
@brief Wrapper function for MoveDiameter
@params index - index of diameter to be moved
@params root_id - index of root to be moved
@params x - x coordinate offset of moved diameter
@params y - y coordinate offset of moved diameter
@returns true for sucess, false for failure
*/
bool Controller::C_MoveDiameterSess(int index, int root_id, int session, int x, int y)
{  
  RootInstance *root_inst = C_GetRootInstance(root_id, session);
  
  /// Failure if root instance is null
  if(root_inst == NULL)     return false;
  
  if(root_inst->MoveDiameter(index, x, y))
  {
    /// Set status unsaved
    Controller::C_SetModify(true);
    Notify(DIAMETER_MOVED);
    return true;
  }
  else return false;
}

/**
@brief The is a wrapper for ChangeWindowOfPoint
@params root_id - index of root instance to which point belongs
@params index - index of point whose window is to be changed
@params w - window to move point to
@returns true for sucess, false for failure
*/
bool Controller::C_ChangeWindowOfDiameter(int index, int root_id, int w)
{
  RootInstance *root_inst = C_GetRootInstance(root_id);
  
  /// Failure if root instance is null
  if(root_inst == NULL)     return false;
  
  if(root_inst->ChangeWindowOfDiameter(index, w))
  {
    /// Set status unsaved
    Controller::C_SetModify(true);
    return true;
  }
  else return false;
}

/**
@brief Wrapper function for ResizeDiameter
@params index - index of diameter to be resized
@params root_id - index of root to be resized
@params radius - new radius of diameter
@returns true for sucess, false for failure
*/
bool Controller::C_ResizeDiameter(int index, int root_id, double radius)
{  
  RootInstance *root_inst = C_GetRootInstance(root_id);
  
  /// Failure if root instance is null
  if(root_inst == NULL)     return false;
  
  if(root_inst->ResizeDiameter(index, radius))
  {
    /// Set status unsaved
    Controller::C_SetModify(true);
    Notify(DIAMETER_CHANGED);
    return true;
  }
  else return false;
}

/**
@brief Wrapper function for ResizeDiameter
@params index - index of diameter to be resized
@params root_id - index of root to be resized
@params radius - new radius of diameter
@returns true for sucess, false for failure
*/
bool Controller::C_ResizeDiameterSess(int index, int root_id, int session, double radius)
{  
  RootInstance *root_inst = C_GetRootInstance(root_id, session);
  
  /// Failure if root instance is null
  if(root_inst == NULL)     return false;
  
  if(root_inst->ResizeDiameter(index, radius))
  {
    /// Set status unsaved
    Controller::C_SetModify(true);
    Notify(DIAMETER_CHANGED);
    return true;
  }
  else return false;
}


/**
@brief Wrapper function for SetSessionNumber
@params session_number - the new session number
@params root_id - index of rootinstance whose session number will be changed
*/
//void Controller::C_SetSessionNumber(int session_number, int root_id)
//{
//  RootInstance *root_inst = C_GetRootInstance(root_id);
//  
//  /// Failure if root instance is null
//  if(root_inst == NULL)
//  {
//    return;
//  }
//  
//  root_inst->SetSessionNumber(session_number);
//  Notify(SESSION_NUMBER_CHANGED);
//}

/**
@brief Wrapper function for CreateRoot
@params primary_window - window in which to create root
@returns Pointer to new root
*/
Root* Controller::C_CreateRoot(int primary_window)
{
  ApplicationData* app_data = m_data.GetApplicationData();
  CString tube_tag = app_data->GetTube();  
  Experiment *exp = m_data.GetExperiment();
  
  Tube* tube = exp->GetTube(tube_tag);
  
  /// If tube does not exist, create a new tube
  if (tube == NULL)  tube = exp->CreateTube(tube_tag);
  assert(tube);
  
  Root* temp_root = tube->CreateRoot(primary_window);
  
  Notify(ROOT_ADDED);
  /// Set status unsaved
  Controller::C_SetModify(true);
  return temp_root;
}


/**
This function copies roots from one session to another.
Either copies only the current tube and window, or all tubes and windows,
depending upon whether 'curr_window' is nonnegative.

  @param from Source session to obtain root information.
  @param to Destination session to receive root information from the specified source.
  @param tubename The tag of the current working tube (ignored if curr_window is -1)
  @param curr_window Specifies the window to perform the copy within, or -1 to copy 
         roots of all windows and all tubes.
*/

void Controller::C_CopySession(int from, int to, const CString &tubename, int curr_window) 
{
  Experiment *exp = m_data.GetExperiment();
  Tube *tube = exp->GetTube(tubename);
  vector<Tube> *alltubes;
  vector<Root> *roots;
  vector<RootInstance> *instances;
  bool copy_all = (curr_window == -1);

  bool notified=false;
  int ok_to_copy = true;    // This is set to false later only if user cancels operation based on message box
  int i,j,k,m;
  
  /// Failure if tube is null
  if(tube == NULL)  return;
  
  //Check the bounds of the source and destination sessions
  assert(from >= 1 && to >= 1);
  if (from < 1 || to < 1) 
  {
    AfxMessageBox("Source and destination sessions must be positive.  Unable to copy.", MB_ICONINFORMATION);
    return;
  }
  assert(from + 1 <= to);
  if (from + 1 > to) 
  {
    AfxMessageBox("Source session must be less than destination session.  Unable to copy.", MB_ICONINFORMATION);
    return;
  }
  
  alltubes = exp->GetAllTubes();
  

  //Check to see if roots exist in destination sessions.  If they are, prompt the user to see
  //if they want them overwritten
  for (i=0; i<alltubes->size(); i++) 
  {
    roots = alltubes->at(i).GetAllRoots();
    
    //Loop through ALL roots in the current tube.
    for (j=0; j<roots->size(); j++)
    {
      if(copy_all || (roots->at(j).GetPrimaryWindow() == curr_window && tube->GetTag() == alltubes->at(i).GetTag()))
      {
        instances = roots->at(j).GetAllRootInstances();
        //Loop through ALL sessions in the current root to determine if there already exists a session
        //in the selected session interval
        for( k=0; k<instances->size(); k++)
        {
          for( m=from+1; m <= to; m++)
          {
            // If the session number of this root instance is equal to l, which is in the interval from "from" to "to"
            //  ask the user if they want to overwrite the destination sessions.
            if( instances->at(k).GetSessionNumber() == m && !notified)
            {
              CString str;
              if (copy_all)
              {
                if (from+1 == to)
                {
                  str.Format("Session %d already contains roots.\r\nAre you sure you want to overwrite this data?", to);
                }
                else
                {
                  str.Format("Sessions %d through %d already contain roots.\r\nAre you sure you want to overwrite this data?", from+1, to);
                }
              }
              else
              {
                if (from+1 == to)
                {
                  str.Format("Window %d of Session %d already contains roots.\r\nAre you sure you want to overwrite this data?", curr_window, to);
                }
                else
                {
                  str.Format("Window %d of Sessions %d through %d already contain roots.\r\nAre you sure you want to overwrite this data?", curr_window, from+1, to);
                }
              }
              DWORD choice = AfxMessageBox(str, MB_YESNO | MB_DEFBUTTON2);
              if (choice != IDYES)  ok_to_copy = false;
              notified = true;
            }
          }
        }
      }
    }
  }


  //If the user chose to overwrite (or wasn't prompted at all) continue
  if (ok_to_copy) 
  {
    //Copy over all tubes and windows
    if (copy_all)
    {
      for (i=0; i<alltubes->size();i++)
      {
        roots = alltubes->at(i).GetAllRoots();
        for (j=0; j < roots->size(); j++) 
        {
          for (k=from+1; k <=to; k++) 
          {
            roots->at(j).DeleteRootInstance(k);
            roots->at(j).CopyRootInstance(from, k);
          }
        }
      }      
    }
    //else only copy within the current window
    else 
    {
      roots = tube->GetAllRoots();
      for(int i = 0; i < roots->size(); i++)
      {
        if(roots->at(i).GetPrimaryWindow() == curr_window)
        {   
          for (k=from+1; k<=to;k++) 
          {
            roots->at(i).DeleteRootInstance(k);
            roots->at(i).CopyRootInstance(from, k);
          }
        }
      }
    }
  }
    
  /// Set status unsaved
  Controller::C_SetModify(true);
  Notify(COPY_PREVIOUS);
}


/**
@brief Wrapper function for CreateTube
@params tag - name of tube to be created
@returns Pointer to new tube
*/
Tube* Controller::C_CreateTube(const CString& tag)
{
  Tube *temp_tube;
  
  Experiment *exp = m_data.GetExperiment();
  temp_tube = exp->CreateTube(tag);
  /// Set so that a root can be added
  temp_tube->SetRootIDCounter(1);
  
  Notify(TUBE_ADDED);
  /// Set status unsaved
  Controller::C_SetModify(true);
  return temp_tube;
}

/**
@brief Wrapper function for DeleteRoot
@params root_id - Index of root to be deleted
@returns true if sucess, false if failure
*/
bool Controller::C_DeleteRoot(int root_id)
{
  bool temp_bool;
  ApplicationData* app_data = m_data.GetApplicationData();
  CString tube_tag = app_data->GetTube();
  
  Experiment *exp = m_data.GetExperiment();
  Tube* tube = exp->GetTube(tube_tag);
  /// Failure if tube is null
  if(tube == NULL) {
    return false;
  }
  
  temp_bool = tube->DeleteRoot(root_id);
  Notify(ROOT_DELETED);
  /// Set status unsaved
  Controller::C_SetModify(true);
  return temp_bool;
}

/**
@brief Wrapper function for SetTube
@params tube - tube to be set as current tube
*/
void Controller::C_SetTube(const CString& tube)
{
  ApplicationData* app_data = m_data.GetApplicationData();
  Experiment* exp = m_data.GetExperiment();
  
  app_data->SetTube(tube);
  /// Reload the image buffers for the new tube
  app_data->ReloadImageBuffers(true,true,exp->GetImageDir(),exp->GetImageFileFormat());
  Notify(TUBE_CHANGED);
}

/**
@brief Wrapper function for SetWindow
@params window - int to change current window to
*/
void Controller::C_SetWindow(int window)
{
  ApplicationData* app_data = m_data.GetApplicationData();
  Experiment* exp = m_data.GetExperiment();
  int changed = window - app_data->GetWindow();
  
  app_data->SetWindow(window);
  /// Update the window buffer for the new window
  app_data->UpdateWindowBuffer(changed,exp->GetImageDir(),exp->GetImageFileFormat());
  Notify(WINDOW_CHANGED);
}

/**
@brief Wrapper function for SetSession
@params session - int to change current session to
*/
void Controller::C_SetSession(int session)
{
  ApplicationData* app_data = m_data.GetApplicationData();
  Experiment* exp = m_data.GetExperiment();
  int changed = session-app_data->GetSession();
  
  app_data->SetSession(session);
  /// Update the session buffer for the new session
  app_data->UpdateSessionBuffer(changed,exp->GetImageDir(),exp->GetImageFileFormat());  
  Notify(SESSION_CHANGED);
}

/**
@brief Wrapper function for SetShowRootID
@params status - Updated status of Show Root IDs checkbox
*/
void Controller::C_SetShowRootID(bool status)
{
  Settings *settings = m_data.GetSettings();
  
  settings->SetShowRootID(status);
  Notify(SHOW_ROOT_ID_CHANGED);
}

/**
@brief Wrapper function for SetShowDeadRoots
@params status - Updated status of Show Dead Roots checkbox
*/
void Controller::C_SetShowDeadRoots(bool status)
{
  Settings *settings = m_data.GetSettings();
  
  settings->SetShowDeadRoots(status);
  Notify(SHOW_DEAD_ROOTS_CHANGED);
}

/**
@brief Wrapper function for SetShowDiameters
@params status - Updated status of Show Diameters checkbox
*/
void Controller::C_SetShowDiameters(bool status)
{
  Settings *settings = m_data.GetSettings();
  
  settings->SetShowDiameters(status);
  Notify(SHOW_DIAMETERS_CHANGED);
}

/**
@brief Wrapper function for SetExperiment
@param experiement - name of new experiment to be set
*/
void Controller::C_SetExperiment(const CString& experiment)
{
  ApplicationData *app_data = m_data.GetApplicationData();
  
  app_data->SetExperiment(experiment);
  Notify(EXPERIMENT_CHANGED);
}

/**
@brief Wrapper function for SetCurrentImage
@params current_image - image to make the current image
*/
void Controller::C_SetCurrentImage(const CString& current_image)
{
  ApplicationData *app_data = m_data.GetApplicationData();
  Experiment *exp = m_data.GetExperiment();
  
  app_data->SetCurrentImage(current_image);
  /// Reload the image buffer for the new image
  app_data->ReloadImageBuffers(true,true,exp->GetImageDir(),exp->GetImageFileFormat());
  Notify(CURRENT_IMAGE_CHANGED);
}

/**
@brief Wrapper function for SetSessionDate
@params date - New session date
*/
void Controller::C_SetSessionDate(const CString& date)
{
  ApplicationData *app_data = m_data.GetApplicationData();
  
  app_data->SetSessionDate(date);
  Notify(SESSION_DATE_CHANGED);
}

/**
@brief Wrapper function for SetOperator
@params operator_initials - New operator initials
*/
void Controller::C_SetOperator(const CString& operator_initials)
{
  ApplicationData *app_data = m_data.GetApplicationData();
  
  app_data->SetOperator(operator_initials);
  Notify(OPERATOR_CHANGED);
}

/**
@brief Sets the save status of the file.  Notifies if the file has gone from saved to unsaved
or from unsaved to saved.  This allows the GUI to know where the file needs to be
or has been saved.
@params modify - true if the file has been changed, false if the file has been saved
*/
void Controller::C_SetModify(bool modify)
{
  ApplicationData *app_data = m_data.GetApplicationData();
  
  /// Get the current save status
  bool temp_status = app_data->GetModify();
  
  /// If the status has changed, update the status in the data scructures and notify
  if(temp_status != modify)
  {
    app_data->SetModify(modify);
    Notify(modify ? SAVE_STATUS_UNSAVED : SAVE_STATUS_SAVED);
  }
}

/**
@brief Wrapper function for SetFilename
@params filename - new filename
*/
void Controller::C_SetFilename(const CString& filename)
{
  ApplicationData *app_data = m_data.GetApplicationData();
  
  app_data->SetFilename(filename);
  /// Fixes an error in which the save status of a file whose name has been changed
  /// will not update save status until changed
  C_SetModify(true);
  Notify(FILENAME_CHANGED);
}

/**
@brief Wrapper function for SetImageDir
@params imgdir - new directory to pull images from
*/
void Controller::C_SetImageDir(const CString& imgdir)
{
  Experiment *expt = m_data.GetExperiment();
  
  expt->SetImageDir(imgdir);
//  C_SetModify(true);
  Notify(IMAGEDIR_CHANGED);
}

/**
@brief Wrapper function for SetImageFileFormat
@params iff - the new imagefileformat
*/
void Controller::C_SetImageFileFormat(ImageFileFormat iff)
{
  Experiment *expt = m_data.GetExperiment();
  expt->SetImageFileFormat(iff);
  Notify(IMAGEFILEFORMAT_CHANGED);
}

/**
@brief Wrapper function for GetImageFileFormat - returns the imagefileformat
@return The Imagefileformat
*/
ImageFileFormat Controller::C_GetImageFileFormat() const
{
  const Experiment *expt = m_data.GetExperiment();
  return expt->GetImageFileFormat();
}

/**
@brief Calls ClearAll for the root instance in the current TWS
@params root_id - ID of rootinstance to clear
@return - true if sucessful, false if unsucessful
*/
bool Controller::C_ClearAllRootInstance(int root_id)
{
  bool temp_bool;
  RootInstance *root_instances = C_GetRootInstance(root_id);
  /// Failure if root instance is null
  if(root_instances == NULL) {
    return false;
  }
  
  temp_bool = root_instances->ClearAll();
  Notify(ROOT_INSTANCE_CLEAR);
  /// Set status unsaved
  Controller::C_SetModify(true);
  return temp_bool;
}

/**
@brief Calls ClearAll for the root in the current TWS
@params root_id - index of root which will be cleared
@return - true if sucessful, false if unsucessful
*/
bool Controller::C_ClearAllRoot(int root_id)
{
  bool temp_bool;
  
  /// Get to the root
  ApplicationData* app_data = m_data.GetApplicationData();
  CString tube_tag = app_data->GetTube();
  Experiment *exp = m_data.GetExperiment();
  Tube* tube = exp->GetTube(tube_tag);
  /// Failure if the tube is null
  if(tube == NULL) {
    return false;
  }
  
  Root* temp_root = tube->GetRoot(root_id);
  /// Failure if the root is null
  if(temp_root == NULL) {
    return false;
  }
  
  temp_bool = temp_root->ClearAll();
  Notify(ROOT_CLEAR);
  /// Set status unsaved
  Controller::C_SetModify(true);
  return temp_bool;
  
}

/**
@brief Calls ClearAll for the current tube (clears everything in the current tube)
@return - True if sucessful, false if unsucessful
*/
bool Controller::C_ClearAllTube()
{
  bool temp_bool;
  
  /// Get to the current tube
  ApplicationData* app_data = m_data.GetApplicationData();
  CString tube_tag = app_data->GetTube();
  Experiment *exp = m_data.GetExperiment();
  Tube* tube = exp->GetTube(tube_tag);
  /// Failure if tube is null
  if(tube == NULL) {
    return false;
  }
  
  temp_bool = tube->ClearAll();
  Notify(TUBE_CLEAR);
  /// Set status unsaved
  Controller::C_SetModify(true);
  return temp_bool;
}

/**
@brief Calls ClearAll for the experiment (clears all information in the current experiment)
@returns true if sucessful, false if unsucessful
*/
bool Controller::C_ClearAllExperiment()
{
  bool temp_bool;
  
  /// Get the Experiment
  Experiment *exp = m_data.GetExperiment();
  
  assert(exp);
  if (exp == NULL)  return false;
  
  temp_bool = exp->ClearAll();
  Notify(EXPERIMENT_CLEAR);
  /// Set status unsaved
//  Controller::C_SetModify(true);  // commented -- only gets called by FileNew -- STB
  return temp_bool;
}

/**
@brief Wrapper function to read settings from registry
*/
void Controller::C_ReadUserPreferences()
{
  /// Get the user preferences
  Settings *set = m_data.GetSettings();
  
  set->ReadUserPreferences();
  Notify(PREFERENCES_READ);
}

/**
@brief Wrapper function to save settings to registry
*/
void Controller::C_SaveUserPreferences()
{
  /// Get the user preferences
  Settings *set = m_data.GetSettings();
  
  set->SaveUserPreferences();
  Notify(PREFERENCES_SAVED);
}

/**
@brief Wrapper function to set max number of windows
@params windows - the new max number of windows
*/
void Controller::C_SetMaxWindows(int windows)
{
  Experiment *exp = m_data.GetExperiment();
  
  exp->SetMaxWindows(windows);
  Notify(MAX_WINDOWS_CHANGED);
}

/**
@brief Wrapper function to set max number of sessions
@params sessions - the new max number of sessions
*/
void Controller::C_SetMaxSessions(int sessions)
{
  Experiment *exp = m_data.GetExperiment();
  
  exp->SetMaxSessions(sessions);
  Notify(MAX_SESSIONS_CHANGED);
}

/**
@brief Wrapper function to set mm conversion
@params conversion - the new mm-to-pixel conversion
*/
void Controller::C_SetMMConversion(float conversion)
{
  Experiment *exp = m_data.GetExperiment();
  
  exp->SetMMConversion(conversion);
  Controller::C_SetModify(true);
  Notify(MM_CONVERSION_CHANGED);
}

/**
@brief Wrapper function to set root drawing color
@params color - the new color for roots being drawn
*/
void Controller::C_SetRootDrawingColor(COLORREF color)
{
  Settings *settings = m_data.GetSettings();
  
  settings->SetRootDrawingColor(color);
  Notify(ROOT_DRAWING_COLOR_CHANGED);
}

/**
@brief Wrapper function to set selected root drawing color
@params color - the new color for the selected root
*/
void Controller::C_SetRootDrawingColorSelected(COLORREF color)
{
  Settings *settings = m_data.GetSettings();
  
  settings->SetRootDrawingColorSelected(color);
  Notify(SELECTED_ROOT_DRAWING_COLOR_CHANGED);
}

/**
@brief Wrapper function to set dead root drawing color
@params color - the new color for dead roots
*/
void Controller::C_SetRootDrawingColorDead(COLORREF color)
{
  Settings *settings = m_data.GetSettings();
  
  settings->SetRootDrawingColorDead(color);
  Notify(SELECTED_ROOT_DRAWING_COLOR_CHANGED);
}

/**
@brief Wrapper function to set dead selected root drawing color
@params color - the new color for dead roots
*/
void Controller::C_SetRootDrawingColorDeadSelected(COLORREF color)
{
  Settings *settings = m_data.GetSettings();
  
  settings->SetRootDrawingColorDeadSelected(color);
  Notify(DEAD_SELECTED_ROOT_DRAWING_COLOR_CHANGED);
}

/**
@brief Wrapper function to set root color
@params root_id - the index of the root whose color is to be changed
@params color_choise - root color to be set
*/
void Controller::C_SetRootColor(int root_id, int color_choice)
{
  ApplicationData *app = m_data.GetApplicationData();
  Experiment *exp = m_data.GetExperiment();
  
  Tube *tube = exp->GetTube(app->GetTube());
  /// Failure if the tube is null
  if(tube == NULL) return;
  Root *root = tube->GetRoot( root_id );
  /// Failure if the root is null
  if(root == NULL) return;
  RootInstance *inst = root->GetRootInstance(app->GetSession());
  /// Failure if the root instance is null
  if(inst == NULL) return;
  if(inst->GetColor() != color_choice)
  {
    /// Set status unsaved
    C_SetModify(true);
    inst->SetColor(color_choice);
    Notify(COLOR_CHANGED);
  }
}

/**
@brief Wrapper function to set selected root status
@params root_id - index of root whose life status is to be changed
@params alive - true if the root is alive, false if the root is dead
*/
void Controller::C_SetRootAlive(int root_id, bool alive)
{
  ApplicationData *app = m_data.GetApplicationData();
  Experiment *exp = m_data.GetExperiment();
  int session = app->GetSession();
  
  Tube *tube = exp->GetTube(app->GetTube());
  /// Failure if the tube is null
  if(tube == NULL) return;
  Root *root = tube->GetRoot( root_id );
  /// Failure if the root is null
  if(root == NULL) return;
  RootInstance *inst = root->GetRootInstance(app->GetSession());
  /// Failure if the root instance is null
  if(inst == NULL) return;
  /// If currently alive and to be set dead
  if(!alive)
  {       
    // sets all instances dead in current and future sessions
    while(session <= exp->GetMaxSessions())
    {
      inst = root->GetRootInstance(session);
      if (inst != NULL && inst->IsAlive())   inst->SetDead();
      session++;
    }
  } 
  /// If currently set dead and to be set alive
  else 
  {
    // sets all instances alive in current and past sessions
    while(session > 0)
    {
      inst = root->GetRootInstance(session);
      if (inst != NULL && !inst->IsAlive())  inst->SetAlive();
      session--;    
    }
  }  
  /// Set status unsaved
  C_SetModify(true);
  Notify(STATUS_CHANGED);
}

/**
@brief Wrapper function to set selected root user defined code
@params root_id - index of root whos user condition code is to be changed
@params condition - user condition code to be attributed to root
*/
void Controller::C_SetConditionCode(int root_id, CString condition)
{
  ApplicationData *app = m_data.GetApplicationData();
  Experiment *exp = m_data.GetExperiment();
  Tube *tube = exp->GetTube(app->GetTube());
  /// Failure if tube is null
  if(tube == NULL) return;
  Root *root = tube->GetRoot( root_id );
  /// Failure if root is null
  if(root == NULL) return;
  RootInstance *inst = root->GetRootInstance(app->GetSession());
  /// Failure if root instance is null
  if(inst == NULL) return;
  CString prev = inst->GetConditionCode();
  /// If the condition code has changed
  if(prev != condition)
  {
    inst->SetConditionCode(condition);
    /// Set status unsaved
//    Controller::C_SetModify(true);  // commented; condition codes are saved in registry, not file -- STB
  }
  
  Notify(CONDITION_CODE_SELECTED);
}

/**
@brief Wrapper function to add user defined code to list
@params condition - user condition code to be added
*/
void Controller::C_AddConditionCode(CString condition)
{
  Settings *set = m_data.GetSettings();
  set->AddConditionCode(condition);
  Controller::C_SetModify(true);
  Notify(CONDITION_CODE_ADDED);
}

/**
@brief Wrapper function to add user defined code to list
@params vector - Vector of condition codes to make the user condition codes
*/
void Controller::C_SetConditionSet(std::vector<CString> vector)
{
  Settings *set = m_data.GetSettings();
  set->AddConditionSet(vector);
//  Controller::C_SetModify(true);  // commented; condition codes are saved in registry, not file -- STB
  Notify(CONDITION_SET_CHANGED);
}

void Controller::C_SetConditionDefaultIndex(int index)
{
  Settings *set = m_data.GetSettings();
  set->SetConditionDefaultIndex(index);
  Notify(CONDITION_DEFAULT_INDEX_CHANGED);
}

/**
@brief Wrapper function to set the session dates for the project
@params dates - Vector of dates specified by the user to represent the sessions
*/

void Controller::C_SetSessionDates(std::vector<Date> dates)
{
  Experiment *exp = m_data.GetExperiment();
  exp->SetSessionDates(dates);
//  Controller::C_SetModify(true);
  Notify(SESSION_DATES_SET);
}
/**
@brief Wrapper function to set current image to a false positive
@params status - bool to indicate whether or not it should be a false positive
*/
void Controller::C_SetImageScanned(bool status)
{
  ApplicationData *app = m_data.GetApplicationData();
  Experiment *exp = m_data.GetExperiment();
  Tube *tube = exp->GetTube(app->GetTube());
  /// Failure if tube is null
  if(tube == NULL) return;
  int curr_window = app->GetWindow();
  int curr_session = app->GetSession();  
  if(tube->GetImageScanned(curr_window,curr_session) != status)
  {
    tube->SetImageScanned(status, curr_window, curr_session);
    C_SetModify(true);
    Notify(IMAGED_SCANNED);
  }
}

int Controller::CalculateDaysBetweenDates(Date start, Date end)
{
  int days_in_month[12] = {31,28,31,30,31,30,31,31,30,31,30,31};
  int num_days = 0;

  // end date is bigger than start date.
  if(end.year < start.year) return -1;
  else if(end.year == start.year && end.month < start.month) return -1;
  else if(end.year == start.year && end.month == start.month && end.day < start.day) return -1;
  // invalid dates, can't calculate
  if(end.month == -1 || start.month == -1) return -1;

  num_days += (end.year-start.year)*365;
  for(int i = end.month-1; i >= start.month; i--)
  {
    num_days += days_in_month[i];
  }
  num_days += end.day;
  num_days -= start.day;
  return num_days;
}

/**
* Handles the export functions for the main application
* @param filename String representing the file to be written
* @return nothing
*/
void Controller::C_Export(CString filename)
{
  
#define DISPLAY_CONDITION_CODE
  
  int i,j,k,l, max_sessions=0, first_dead_session=0, first_alive_session=0, lifespan=0;
  CString tubename, color, status;        //Status == Alive/Dead
  double length=0, avg_diam=0, max_length;
  float conversion;
  const std::vector<Tube> *tubes;
  const std::vector<Root> *roots;
  const std::vector<RootInstance> *instances;
  vector<int> window_numbers;
  Date start, end;
  Experiment *exp = m_data.GetExperiment();;
  FILE *fp = fopen(filename, "wt");  
  //exp 
  tubes=exp->GetAllTubes();
  conversion = exp->GetMMConversion();
  
  if(fp == NULL) 
  {
    AfxMessageBox("Unable to open " + filename + " for writing");
    return;
  }
  
  if (tubes == NULL) 
  {
    return;
  }
  
  //Determine the maximum number of sessions in this experiment.
  for(i=0; i<tubes->size(); i++) 
  {
    roots = tubes->at(i).GetAllRoots();
    for(j=0; j<roots->size(); j++) 
    {
      if(roots->at(i).GetLastSessionNumber() > max_sessions) 
      {
        max_sessions = roots->at(i).GetLastSessionNumber();
      }
    }
  }
  
  //Output columns
  fprintf(fp, "Tube,Window,RootID,Appeared,Died,Lifespan,Maximum Length,Maximum Diameter,");
  for(i=0; i<=max_sessions; i++) 
  {
    
#ifdef DISPLAY_CONDITION_CODE
    fprintf(fp, "Length date(%d),Diameter date(%d),Color date(%d),Condition Code date(%d),",i+1,i+1,i+1,i+1);
#else
    fprintf(fp, "Length date(%d),Diameter date(%d),Color date(%d)",i+1,i+1,i+1);
#endif
  }
  
  fprintf(fp, "\n");
  
  //Output the data, one root per line.
  for(i=0; i< tubes->size(); i++) 
  {
    for(j=0; j< roots->size(); j++) 
    {      
      //if(tubes->at(i).GetTag() == "") continue;
      first_dead_session = roots->at(j).GetFirstDeadSessionNumber();
      first_alive_session = roots->at(j).GetFirstSessionNumber();
      
      //If there are rootsessions available within this root....
      if(first_alive_session != -1 && first_dead_session != -1) 
      {
        
        //Output initial root information
        fprintf(fp, "%s,%d,%d,%d,", tubes->at(i).GetTag(), roots->at(j).GetPrimaryWindow(), roots->at(j).GetWindowID(), first_alive_session);
        
        //Make sure the values for the lifespan are the correct values.  These functions can return -1 and -2, which
        //  need to be handled to give this value the correct meaning.
        if(first_dead_session == -2) 
        {                              
          start = exp->GetSessionDate(first_alive_session);
          end = exp->GetSessionDate(roots->at(j).GetLastSessionNumber());
          lifespan = CalculateDaysBetweenDates(start, end);
          fprintf(fp, "-,%d,", lifespan);
        }
        else
        {
          start = exp->GetSessionDate(first_alive_session);
          end = exp->GetSessionDate(first_dead_session);
          lifespan = CalculateDaysBetweenDates(start, end);
          fprintf(fp, "%d,%d,", first_dead_session, lifespan);
        }
        
        //Output the maximum length of the root and the maximum diameter on the root.
        fprintf(fp, "%f,%f",  roots->at(j).GetMaxLength(conversion), roots->at(j).GetMaxDiameter(conversion));
        
        //Start outputting where the root first exists (done by session number)
        for(k=1; k < roots->at(j).GetFirstSessionNumber(); k++)
        {
#ifdef DISPLAY_CONDITION_CODE
          fprintf(fp, ", , , ,");
#else
          fprintf(fp, ", , ,");
#endif
        }
        
        //Start outputting session specific information: color, length, diameter, and possibly the condition code
        instances = roots->at(j).GetAllRootInstances();
        
        //Pick the appropriate color to output to the file
        for(k=0; k < instances->size(); k++) 
        {
//          TRACE("\ninstances->size():  %d",instances->size());
          max_length = 0.0;
          switch(instances->at(k).GetColor())
          {
          case 0:	  color = "White";       break;
          case 1:   color = "Light Brown"; break;
          case 2:   color = "Dark Brown";  break;
          case 3:   color = "Black";       break;
          default:  color = "White";       break;
          }
          instances->at(k).GetWindowNumbers(&window_numbers);	
          
          //Find the maximum length
          for(l=0; l < window_numbers.size(); l++) 
          {
            max_length += instances->at(k).FindLengthInWindow(window_numbers.at(l), conversion);
          }
          
          //Output the root per-session information
#ifdef DISPLAY_CONDITION_CODE
          fprintf(fp, ",%f, %f,%s,%s", max_length, instances->at(k).GetAverageDiameter(conversion), color, instances->at(k).GetConditionCode());
//          TRACE("\n%f, %f,%s,%s", max_length, instances->at(k).GetAverageDiameter(conversion), color, instances->at(k).GetConditionCode());
#else
          fprintf(fp, ",%f, %f,%s", max_length, instances->at(k).GetAverageDiameter(conversion), color);
//          TRACE("\n%f, %f,%s", max_length, instances->at(k).GetAverageDiameter(conversion), color);
#endif
          
        }
        fprintf(fp,"\n");
      }
      
    }
  }
  fclose(fp);
}


/**
@brief Wrapper function to set the default file directory
@params dir - CString that is the diectory to be set to the default directory
*/
void Controller::C_SetFileDirectory(CString dir)
{
  Settings *set = m_data.GetSettings();
  set->SetFileDir(dir);
  Notify(DEFAULT_DIR_CHANGED);
}

/**
@brief Wrapper function to convert root coordinates
*/
void Controller::C_ConvertOldRoots(const CString& dir, ImageFileFormat& image_file_format)
{
  Experiment *exp = m_data.GetExperiment();
  exp->ConvertOldRoots(dir, image_file_format);
}

// has progress bar
void Controller::C_ConvertOldRoots(const CString& dir, ImageFileFormat& image_file_format, CWnd *pParentWnd)
{
  Experiment *exp = m_data.GetExperiment();
  exp->ConvertOldRoots(dir, image_file_format, pParentWnd);
}
