/*
* Rootfly:  A program for analyzing and measuring roots.
* Copyright (c) 2007 Clemson University.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.

* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// $Id: DataStructures.h,v 1.4 2008/02/27 16:37:52 stb Exp $

#include "Image.h"
#include "ImageFileFormat.h"
#include "stdafx.h"
#include <vector>
#include <iterator>
#include "version1_files/RootflyData.h"


#if _MSC_VER > 1020      // if VC++ version is > 4.2
using namespace std;  // std c++ libs implemented in std
#endif

#pragma warning(disable:4786)  //Added to remove compiler warning **Corey Dickerson

#ifndef __DATASTRUCTURES__H__
#define __DATASTRUCTURES__H__

namespace ERRORS
{  
  typedef enum 
  {
    RFY_LOADERROR_SUCCESS = 0,
    RFY_LOADERROR_FILECORRUPT = 1,
    RFY_LOADERROR_VERSION = 2,
    RFY_LOADERROR_NOFILEFOUND = 3,
    RFY_LOADERROR_USERCANCELED = 4
  } DATASTRUCTURE_ERROR;
}

enum Mode
  {
      MODE_NOFILELOADED,			  // when app first starts up
      MODE_NOIMAGEDISPLAYED,		// no image for the current Tube/window/session
      MODE_IMAGEDISPLAYED,      // image displayed
      MODE_DRAWINGNEWROOT,      // drawing a new root (before first point clicked)
      MODE_RESIZINGDIAMETER,    // resizing diameter      
      MODE_EXTENDINGENDPOINT,   // extending endpoint of selected root
      MODE_LOCATINGDIAMETER,    // locating position of diameter for new root
      MODE_ROOTSELECTED,
      MODE_MOVINGRPOINT,        // moving point on selected root
      MODE_STARTINGMOVEALL,
      MODE_MOVINGALLROOTS,      // moving all roots
      MODE_MOVINGSINGLEROOT,    // moving single root 
      MODE_REPOSITIONDIAMETER,  // moving diameter of selected root
      MODE_SIZINGDIAMETER       // changing size of diameter for new root
};

/**
@struct RPoint
  
  @brief Structure responsible for holding (x,y) coordinate and window number

  This is the coordinates for the root containing position and window.

*/
struct RPoint
{
public:
  RPoint();
  RPoint( int x, int y, int window );
  void Copy(RPoint &point2);  
  int x;
  int y;
  int window;
};

/**
@struct ImageScanned
  @brief Structure responsible for holding the window and session of a 
  scanned image.
  This is when the automatic detection runs on the image.
  This is the location of that scan.
*/
struct ImageScanned
{
public:
  ImageScanned() { window = 0; session = 0; }
  ImageScanned(int w, int s) { window = w; session = s ; }
  int window;
  int session;
};

/** @struct RDiameter
@brief Contains the root diameter

  Contains the radius measurement and the point of the start of the radius.
*/
struct RDiameter
{
public:
  RDiameter();
  RDiameter( double rad, RPoint& rp );
  RDiameter( double rad, const RPoint& rp );
  void Copy(RDiameter &diam2);
  double radius;
  RPoint r_point;
};


/**@class RootInstance

 @brief This class holds all the data for a single root including the window and coordinates.
*/
class RootInstance
{
public:
  enum Color { COLOR_WHITE, COLOR_LIGHTBROWN, COLOR_DARKBROWN, COLOR_BLACK };
  
  RootInstance();         // empty constructor does not copy from previous session
  virtual ~RootInstance();  
  void Copy(RootInstance &inst2) ;  
  const std::vector<RPoint>* GetAllPoints() const { return &m_points; }   // get all points from m_root vector  
  std::vector<RPoint>* GetAllPoints() { return &m_points; }
  RPoint* GetPoint(int index) { assert(index>=0 && index<m_points.size());  return &m_points[index]; }
  const RPoint* GetPoint(int index) const{ assert(index>=0 && index<m_points.size());  return &m_points[index]; }
  RPoint* CreatePoint();
  bool DeletePoint(int index);
  bool MovePoint(int index, int x, int y);
  bool ChangeWindowOfPoint(int index, int w);
  bool MoveDiameter(int index, int x, int y);
  bool ChangeWindowOfDiameter(int index, int w);
  bool ResizeDiameter(int index, double radius);
  
  const std::vector<RDiameter>* GetAllDiameters() const { return &m_diameters; }   // get all diamters from m_diameter vector  
  std::vector<RDiameter>* GetAllDiameters() { return &m_diameters; }   // get all diamters from m_diameter vector  
  RDiameter* GetDiameter(int index) { assert(index>=0 && index<m_diameters.size());  return &m_diameters[index]; }
  const RDiameter* GetDiameter(int index) const { assert(index>=0 && index<m_diameters.size());  return &m_diameters[index]; }
  RDiameter* CreateDiameter();
  bool DeleteDiameter(int index);

  const Color GetColor() const { return m_color; }
  void SetColor(int color);
  const CString GetConditionCode() const { return m_condition_code; }
  void SetConditionCode(const CString &condition_code) { m_condition_code = condition_code; }
  const int GetSessionNumber() const { return m_session_number; }
  void SetSessionNumber(int session_number) { m_session_number = session_number; }
  bool IsAlive() { return m_is_alive; }
  const bool IsAlive() const { return m_is_alive; }
  void SetAlive() { m_is_alive = true; }
  void SetDead() { m_is_alive = false; }

  bool ClearAll();         //Clears the vectors doesn't touch session#,color,condition, or live

  bool IsRootInWindow(int window_number) const
  {
    for (int i=0 ; i<m_points.size() ; i++)
    {
      if (m_points[i].window == window_number)  return true;
    }
    return false;
  }
  
  //Methods for Export
  void GetWindowNumbers(vector<int>* window_list) const;
  const double FindLengthInWindow(int window, float conversion) const;
  const double GetAverageDiameter(float conversion)const;

  // file format group
  int Load(FILE *fpt);                      // Loads a session, returns the RootInstance - GHE
  int Save(FILE *fp) const;                       //Should return 0 for success, -1 for failure.

  bool operator<(const RootInstance& other) { return m_session_number < other.m_session_number; }

private:
  std::vector<RPoint> m_points;
  std::vector<RDiameter> m_diameters;
  Color m_color;
  CString m_condition_code;
  int m_session_number;
  bool m_is_alive;
};



/**
@class Root
  @brief This class holds the data for an individual root over time.
*/
class Root
{
  
public:
  Root();
  Root(int ID);
  virtual ~Root();

  // Copies this root out to 'root2'
  void Copy(Root &root2);

  // Returns a pointer to the root instance in a particular session;
  // returns NULL if no such instance exists
  RootInstance* GetRootInstance(int session_number);
  const RootInstance* GetRootInstance(int session_number) const;

  // Creates a root instance in a particular session, and returns a pointer
  // to the newly created root instance.
  // If a root instance already exists for the session, then no root instance is created;
  // instead, a pointer to the existing root instance is returned.
  RootInstance* CreateRootInstance(int session_number);
  RootInstance* CreateRootInstance();
  // Deletes a root instance in a particular session
  bool DeleteRootInstance(int session_number);

  //This is the implementation of CopyPrevious for the root objects.  'from' and 'to' are session numbers.
  bool CopyRootInstance(int from, int to);
  
  // Get all roots from m_instances vector  
  const std::vector<RootInstance>* GetAllRootInstances() const { return &m_instances; }
  std::vector<RootInstance>* GetAllRootInstances() { return &m_instances; }

  bool ClearAll();       //Clears the vector 

  // file format group
  int Load(FILE *fpt);                      // Loads a root, returns the Root - GHE
  int Save(FILE *fp) const;                       //Should return 0 for success, -1 for failure.

  // gets and sets the root id (starts at 1 for each tube)
  int GetID() const { return m_id; }
  void SetID(int id) { m_id = id; } 
  
  // gets and sets the primary window (the start) of the root
  int GetPrimaryWindow() const { return m_primary_window; }
  void SetPrimaryWindow(int w) { m_primary_window = w; }

  //Methods used for copy previous and export functions
  const int GetLastSessionNumber() const;               //Returns the largest session number found in the vector of rootinstances.
  const int GetFirstSessionNumber() const;              //Returns the smallest session number found in the vector of root instances.
  const int GetFirstDeadSessionNumber() const;          //Returns the first session where this root was murdered.
  const double GetMaxLength(float conversion) const;    //Returns the longest root length out of all the sessions.
  const double GetMaxDiameter(float conversion) const;  //Returns the largest diameter size out of all the sessions.

  // gets and sets the window id (starts at 1 for each window in this tube)
  const int GetWindowID() const { return m_window_id; }
  int GetWindowID() { return m_window_id; }
  void SetWindowID(int win_id) { m_window_id = win_id; }

private:
  std::vector<RootInstance> m_instances; // root instances sorted in ascending order by session number
  int m_id;                              // unique id for entire tube
  int m_window_id;                       // unique id for each window
  int m_primary_window;                  // root's window origin
};

/**
@class Tag
@brief This class contains the methods for a tag

  The class contains methods for setting/getting a tag, minchar, maxchar, and the ++ and -- operators
*/
class Tag
{
public:
  Tag operator++(int);          // increment tag - JHP
  Tag operator--(int);          // decrement tag - JHP
  void SetTag(CString tag);     // set tag - JHP
  CString GetTag() const;       // get tag - JHP
  void Setminchar(TCHAR min);   // Set minimum character for tag - JHP
  TCHAR Getminchar() const;     // returns minimum charater for tag - JHP
  void Setmaxchar(TCHAR max);   // Set maximum character for tag - JHP
  TCHAR Getmaxchar() const;     // returns maximum charater for tag - JHP
private:
  CString m_tag;                // tag stored as CSTRING - JHP
  TCHAR minchar;                // minimum character used in tube tag - JHP
  TCHAR maxchar;                // maximum character used in tube tag - JHP
};

/**
@class Tube

  @brief This class contains the data for a tube.

  This class holds all the Roots. A root contains information about the entire existance for a given root.
*/
class Tube
{
public:
  Tube();
  Tube(const CString& tag);
  virtual ~Tube();
  
  const std::vector<Root>* GetAllRoots() const { return &m_root; }   // get all roots from m_root vector
  std::vector<Root>* GetAllRoots() { return &m_root; }   // get all roots from m_root vector, NEW!! non-const edition! 
  std::vector<Root>* GetRootsInWindow(int window);
  Root* GetRoot(int ID);
  const Root* GetRoot(int index) const;
  Root* CreateRoot();
  Root* CreateRoot(int window);
  bool DeleteRoot(int index);
  
  void SetTubeTag(const CString& tag); // set tube with specified tag
  CString GetTag() const;

  void SetRootIDCounter(const int next){m_next_root_id = next;}
  const int GetRootIDCounter() const {return m_next_root_id;}
  int GetNextRootWindowID(int primary_window);

  void SetImageScanned(bool status, int window, int session);   // sets an image as scanned
  const bool GetImageScanned(int window, int session)const;     // returns true if the current tube, window, 
                                                                // session has been scanned, false otherwise

  //const int GetNextRootWindowID(int primaryWindow);

  bool ClearAll();                 // Clears the vector doesn't change the tag
  
  // file format group
  int Load(FILE *fpt);             // Loads a tube, returns the Tube - GHE
  int Save(FILE *fp) const;              // Should return 0 for success, -1 for failure.

private:  
  vector<ImageScanned> m_image_scanned; // vector that hold which images are scanned
  vector<Root> m_root;             // vector that holds all the roots
  vector<Root> m_roots_in_window;  // place to store roots for move all
  CString m_tag;                   // optional string identifier that could be queried
  int m_next_root_id;   
};


/**
@class Settings

 @brief This class holds all of the permanent user prefences that won't change from session to session.

  This class contains user preferences for displaying such as the GUI checkboxes, the color, and the last window location.
*/
class Settings {
  
public:
  Settings();
  ~Settings();

  // Returns the version of the program; the version number is compiled into 
  // the executable and cannot be changed.
  static CString GetVersion() { return m_version; }

  //void SetRfyFile(CString filename) { m_filename = filename; }
  //CString GetRfyFile() { return m_filename; }

 // bool SetModified( bool status = true ) { m_modified = status; }
 // bool WasModified() { return m_modified; }

  void SetShowRootID(bool status)    {m_show_root_ids = status; }
  void SetShowDeadRoots(bool status) {m_show_dead_roots = status; }
  void SetShowDiameters(bool status) {m_show_diameters = status; }

  const bool RootIdIsEnabled() const { return m_show_root_ids; }
  const bool DeadRootsIsEnabled() const { return m_show_dead_roots; }
  const bool ShowDiametersIsEnabled() const { return m_show_diameters; }

  void SetRootDrawingColor(COLORREF ref)  {m_root_drawing_color = ref; }
  void SetRootDrawingColorSelected(COLORREF ref) {m_root_drawing_color_selected = ref;}
  void SetRootDrawingColorDead(COLORREF ref) {m_root_drawing_color_dead = ref;}
  void SetRootDrawingColorDeadSelected(COLORREF ref) {m_root_drawing_color_dead_selected = ref;}

  COLORREF GetRootDrawingColor() const { return m_root_drawing_color; }
  COLORREF GetRootDrawingColorSelected() const { return m_root_drawing_color_selected; }
  COLORREF GetRootDrawingColorDead() const { return m_root_drawing_color_dead; }
  COLORREF GetRootDrawingColorDeadSelected() const { return m_root_drawing_color_dead_selected; }
  
  COLORREF GetDefaultRootDrawingColor() const { return m_default_root_drawing_color; }
  COLORREF GetDefaultRootDrawingColorSelected() const { return m_default_root_drawing_color_selected; }
  COLORREF GetDefaultRootDrawingColorDead() const { return m_default_root_drawing_color_dead; }
  COLORREF GetDefaultRootDrawingColorDeadSelected() const { return m_default_root_drawing_color_dead_selected; }
  
  void SetLastWinLocation(CRect loc){ m_last_app_window_location = loc;}
  CRect GetLastWinLocation(){ return m_last_app_window_location;}

  void AddConditionCode(CString code){ m_condition_vector.push_back(code);}
  int GetConditionSize() { m_condition_vector.size();}
  CString GetConditionCode(int index) const { return m_condition_vector.at(index);}
  const std::vector<CString> GetConditionSet() const { return m_condition_vector;}
  void AddConditionSet(std::vector<CString> set) { m_condition_vector = set;}  
  const int GetConditionDefaultIndex() const { return m_condition_default_index;}
  void SetConditionDefaultIndex(int index) { m_condition_default_index = index; }  
  

  // sets the default file directory
  const CString GetFileDir() const { return m_file_dir; }
  void SetFileDir(CString dir) { m_file_dir = dir; }

  // saves and loads from the registry
  void SaveUserPreferences();
  void ReadUserPreferences();
  
private:
  COLORREF m_root_drawing_color;
  COLORREF m_root_drawing_color_selected;
  COLORREF m_root_drawing_color_dead;
  COLORREF m_root_drawing_color_dead_selected;
  static const COLORREF m_default_root_drawing_color;
  static const COLORREF m_default_root_drawing_color_selected;
  static const COLORREF m_default_root_drawing_color_dead;
  static const COLORREF m_default_root_drawing_color_dead_selected;
  CRect m_last_app_window_location;  
  CString m_file_dir;
  int m_condition_default_index;

  // GUI checkboxes
  bool m_show_root_ids;
  bool m_show_dead_roots;
  bool m_show_diameters;
  static const CString m_version;  // version of the program
  std::vector<CString> m_condition_vector;  
};



/**
@struct CurrentImageProperties
@brief This struct contains all the data available in a Bartz filename.

  This struct contains all the data available in a Bartz filename.If Bartz is not used, then some of the values may be uninteresting.

*/
struct CurrentImageProperties
{
  CString m_experiment_name;
  CString m_current_image_filename;
  CString m_session_date;
  CString m_operator_initials;
  CString m_tube; // needs to be a string to support tubes like 001A, 001B,...
  int m_window;
  int m_session;
};

  
/**
@class ApplicationData
@brief This class holds all the data relevant to the current image displayed
  and position in data structure.

  The ApplicationData class contains all the temporary data that is held during a session but no
  longer. This data includes the current filename, modified status, current image displayed and
  all associated properties, and the image cache for the thumbnails.
*/
class ApplicationData
{
public:
  ApplicationData();
  ~ApplicationData();

  CString GetExperiment()   const { return m_curr.m_experiment_name; }
  CString GetCurrentImage() const { return m_curr.m_current_image_filename; }
  CString GetSessionDate()  const { return m_curr.m_session_date; }
  CString GetOperator()     const { return m_curr.m_operator_initials; }

  CString GetTube()         const { return m_curr.m_tube; }
  const int GetWindow()     const { return m_curr.m_window; }
  const int GetSession()    const { return m_curr.m_session; }

  bool GetModify()          const { return m_modified; }
  CString GetFileName()     const { return m_filename; }

  const blepo::ImgBgr* getMainImage() { return &m_main_image; }

  void SetExperiment(const CString& experiment) { m_curr.m_experiment_name = experiment; }
  void SetCurrentImage(const CString& current_image) { m_curr.m_current_image_filename = current_image; }
  void SetSessionDate(const CString& date) { m_curr.m_session_date = date; }
  void SetOperator(const CString& operator_initials) { m_curr.m_operator_initials = operator_initials; }

  void SetTube(const CString& tube) { m_curr.m_tube = tube; }
  void SetWindow(int window) { m_curr.m_window = window; }
  void SetSession(int session) { m_curr.m_session = session; }

  void SetModify(bool modify) { m_modified = modify; }
  void SetFilename(const CString& filename) { m_filename = filename; }

  void ReloadImageBuffers(bool reload_window, bool reload_session,CString dir, ImageFileFormat file_format);
  void UpdateWindowBuffer(int changed, CString dir, ImageFileFormat file_format);
  void UpdateSessionBuffer(int changed, CString dir, ImageFileFormat file_format);
  const blepo::ImgBgr GetWindowImage(int index) const { assert(index>=0 && index<5);  return m_window_buffer[index]; };
  const blepo::ImgBgr GetSessionImage(int index) const { assert(index>=0 && index<5);  return m_session_buffer[index]; };
  CString GetSessionDate(int index) const { assert(index>=0 && index<5);  return m_app_session_dates[index]; };
  bool ClearAll();

private:
  bool m_modified;                 // whether current file has been modified since the last save
  CString m_filename;              // .rfy file name
  CurrentImageProperties m_curr;   // struct holding image properties
  enum { NTHUMBNAILS = 5 };       // number of thumbnails

  blepo::ImgBgr m_window_buffer[NTHUMBNAILS];             // image buffer holding images for each window for a particular session within a tube
  blepo::ImgBgr m_session_buffer[NTHUMBNAILS];            // image buffer holding images for each session for a particular window within a tube  
  CString m_app_session_dates[NTHUMBNAILS];                   // dates for each picture in the session buffer
  
  // image buffer holding images for each session for a particular window within a tube
  blepo::ImgBgr m_main_image;                   // currently viewed window image
  blepo::ImgBgr m_temp_image;                   // image data for image loaded from outside cache
};

/**
@class Experiment
@brief This class holds all the tubes in the experiment.

  This class contains the functions that manipulate the experiment data.
*/
class Experiment {
  
public:
  
  Experiment();
  virtual ~Experiment();
  

  void SetAppData(ApplicationData &app_data) { m_app_data = &app_data; };
  int GetNTubes() const;                              // returns number of tubes in experiment
  Tube* GetTube(const CString& tag);                  // searches vector for matching tag
  const Tube* GetTube(const CString& tag) const;      // searches vector for matching tag
  const std::vector<Tube>* GetAllTubes() const { return &m_tubes; };       // returns all the tubes from m_tubes
  std::vector<Tube>* GetAllTubes() { return &m_tubes; };       // returns all the tubes from m_tubes


///JTM modified 3/3/07   = _T("default")  
  Tube* CreateTube( const CString& tag );               // creates a tube with the specified tag
  bool DeleteTube(const CString& tag);                // removes tube with specified tag in the m_tubes vector

  bool SetImageFormat( CString format );
  CString GetImageFormat() const;
  
  /// New ImageFormat functions - msb
  void SetImageFileFormat(ImageFileFormat iff) { m_image_file_format = iff; }
  ImageFileFormat GetImageFileFormat() const { return m_image_file_format; }

  ///SetImageDir
  bool SetImageDir( CString directory );
  const CString GetImageDir() const;
  
  bool SetMMConversion( float mm_conversion );
  float GetMMConversion() const;

  int Load(char *filename, Settings *parameters);    // Loads information from a given filename.  Returns -1 on failure, 0 on success.
  int Save(char *filename, Settings *parameters) const;     // Recursively saves information from all data structures.  Returns -1 on failure, 0 on success.
  
  bool ClearAll();

  const int GetMaxWindows() const { return m_max_windows; }
  const int GetMaxSessions() const { return m_max_sessions; }
  void SetMaxWindows(int windows=5) { m_max_windows = windows; } // default value 5
  void SetMaxSessions(int sessions=5) { m_max_sessions = sessions; } // default value 5

  // Sets the current status of the experiment.
  // If the experiment was a converted old file
  // the status should be set true.  If the file
  // is a normal new file, status should be false.
  void Experiment::SetOldFile(bool status){ m_is_old;}

  // Returns true if the current experiment was a converted old file
  const bool Experiment::IsOldFile() const { return m_is_old; }

  int Experiment::ConvertDataStructure(OldData::RootflyData oldStructure, Settings *settings, const CString &filename);

  // Converts version 1 roots (points and diameters) from client coordinates (the old way)
  // to the proper image coordinates (the new way).
  void ConvertOldRoots(const CString& dir, ImageFileFormat& image_file_format);
  void ConvertOldRoots(const CString& dir, ImageFileFormat& image_file_format, CWnd *pParentWnd);

  // Converts version 1 points to the proper image coordinates
  //  void Experiment::ConvertOldPoints(const CRect& image_rect);

  // sets session dates for the experiment
  void SetSessionDates(std::vector<Date> dates) { m_session_dates = dates; }

  // returns the session dates vector
  const std::vector<Date> GetSessionDates() const { return m_session_dates; }

  // returns the session date for the given session
  const Date GetSessionDate(int index) const 
  { 
    Date session; 
    if (index < 1 || index > m_session_dates.size() )
    { 
      session.day = -1;
      session.month = -1; 
      session.year = -1; 
    } 
    else 
    {
      session = m_session_dates.at(index-1);
    }  
    return session; 
  }
  
protected:
  int LoadOld(char *filename, Settings *parameters);  // Loads information from an old file format
  

private:
  ApplicationData *m_app_data;
  std::vector<Tube> m_tubes;           // vector of tubes for experiment
  std::vector<Date> m_session_dates; // a vector of session_dates
  CString m_image_format;              // image file format  
  ImageFileFormat m_image_file_format; // new image file format - msb
  CString m_image_dir;                 // image directory
  float m_mm_conversion;               // millimeter conversion ratio
  int m_max_windows;                   // largest quantity of windows
  int m_max_sessions;                  // largest quantity of sessions
  bool m_is_old;               // Current experiment is a converted old file; set to true temporarily until user selects image directory so that points can be converted to image coordinates
};


/**
@class RootflyData

 @brief This class holds an experiment and all the associated settings and application data.

  This is the "guts" of the project and top level of the data structure.
  RootflyData contains the experiment, settings, and application data.

  The experiment has a vector of tubes. Each tube has a vector of roots.
  Each root has a vector of root instances (sessions). Each root instance has a
  vector of points and diameters.

  The application data has the current session properties as well as contains a struct of the
  current image properties.        
*/
class RootflyData
{
  
public:
  RootflyData();
  ~RootflyData();
  Experiment* GetExperiment() { return &m_exp; }                           // this returns the object holding all the tubes
  const Experiment* GetExperiment() const { return &m_exp; }               // this returns the object holding all the tubes
  Settings* GetSettings() { return &m_settings; }                          // this returns the settings object with all the user settings
  const Settings* GetSettings() const { return &m_settings; }              // this returns the settings object with all the user settings
  ApplicationData* GetApplicationData() { return &m_appdata; }             // this returns the object holding the ApplicationData
  const ApplicationData* GetApplicationData() const { return &m_appdata; } // this returns the object holding the ApplicationData
  
private:
  ApplicationData m_appdata;    // temporary data, disappears after closing application
  Experiment m_exp;             // stored in .rfy file
  Settings m_settings;          // stored in Windows registry
};



/**
 @class Registry
 @brief This class allows one to read and write to the registry.

  This class allows one to read and write strings and integers to and from the registry.
  
  Be sure to call CWinApp::SetRegistryKey method first to set the base key for a project. It is currently set
  in the Rootfly.cpp file so that the CWinApp object knows to write to the registry.
*/

class Registry
{

public:
  Registry(const CString& section) { pApp = AfxGetApp();  m_section = section; }
  ~Registry() { }; // empty destructor

  // string operations
  void SaveStringToReg(const CString& name, const CString& data);
  CString GetStringFromReg(const CString& name) const;

  // integer operations
  void SaveIntToReg(const CString& name, const int& data);
  int GetIntFromReg(const CString& name) const;

private:
  CWinApp* pApp;
  CString m_section;
};

class DebugLogFile
{
public:
  DebugLogFile() : m_file(NULL) {}
  ~DebugLogFile() { Close(); }
  void Open(const char* fname) { if (m_file==NULL) { m_file = fopen(fname, "wt"); } }
  void Close() { if (m_file) fclose(m_file);  m_file = NULL; }
//  void Print(const char* msg) { if (m_file)  { fprintf(m_file, msg); fflush(m_file); } }
  void Print(const char* format, ...)
  {
    if (m_file)
    {
      va_list arglist;
      va_start(arglist, format);													    
      CString str;
      str.FormatV(format, arglist);															
      va_end(arglist);
      fprintf(m_file, "%s", (const char*) str);
      fflush(m_file);
    }
  }
private:
  FILE* m_file;
};

#endif __DATASTRUCTURES__H__
